//////////////////////////////////////////////////////////////////////////////////////////////////
//Programmer:		Jason Bouchard
//Project:		Project 2
//Course:		CS 343
//Description:   
//			Recursive - Descent Parser to parse and evaluate arithmetic expressions 
//			involving integer constants.	
//
//Compile Instructions:  javac Parser.java
//Execution Instructions: Non executable.  No main method.
//////////////////////////////////////////////////////////////////////////////////////////////////


import java.lang.*;

public class Parser{
		Scanner scanner;
		boolean trace = false;

		//set and get methods for variable trace.
		public void setTrace(boolean value)
		{
		
		trace = value;
		
		}
		
		public boolean getTrace()
		{
		
		return trace;

		}

		public int Parse(String line) throws ParseException
		{
			//Getting the new text and checking whether we've reached the end of the string.
			scanner = new Scanner(line);
			scanner.NextToken();

			if (trace && scanner.CurrentToken().gettkType() != Token.EOL)
			{
				System.out.println(scanner.CurrentToken().ToString());
			}
			return Command();
		}

		private int Command() throws ParseException
		{
			//command -> expr '\n'
			int result = Expr();
			if (scanner.CurrentToken().gettkType() == Token.EOL)
			{
				return result;
			}
			else
			{
				
				throw new ParseException("Parse Error: Command Error");
			}

		}

		private void Match(int tkType)  throws ParseException
		{
			//if token type equals the current token's type.
			if (tkType == scanner.CurrentToken().gettkType())
			{
				//get the next token.
				scanner.NextToken();

				if (trace && scanner.CurrentToken().gettkType() != Token.EOL)
				{
					System.out.println(scanner.CurrentToken().ToString());
				}
			}
			else
			{
				//if we have gotten this far, we have an error.
				throw new ParseException("Parse Error: Match Error");
			}
		}

		private int Expr()  throws ParseException
		{
			//expr -> term{+term | -term}
			int result = Term();
			while (scanner.CurrentToken().gettkType() == Token.PLUS || 
						scanner.CurrentToken().gettkType() == Token.MINUS) 
			{
				if (scanner.CurrentToken().gettkType() == Token.PLUS) 
				{
					Match(Token.PLUS);
					result += Term();
				} 
				else 
				{
					Match(Token.MINUS);
					result -= Term();
				}
			}
			return result;
		}

		private int Term()  throws ParseException
		{
			//term -> power {*power|/power|%power}
			int result = Power();
			while (scanner.CurrentToken().gettkType() == Token.MULT 
						|| scanner.CurrentToken().gettkType() == Token.DIVIDE 
						|| scanner.CurrentToken().gettkType() == Token.REMAINDER) 
			{
				if (scanner.CurrentToken().gettkType() == Token.MULT) 
				{
					Match(Token.MULT);
					result *= Power();
				} 
				else if (scanner.CurrentToken().gettkType()== Token.DIVIDE) 
				{
					Match(Token.DIVIDE);
					result /= Power();
				} 
				else 
				{
					Match(Token.REMAINDER);
					result %= Power();
				}
			}
			return result;
		}

		private int Power()  throws ParseException
		{
			//power -> factor[^power]
			int result = Factor();
			int pow, temp;

			if (scanner.CurrentToken().gettkType() == Token.POWER) 
			{
				Match(Token.POWER);
				pow = Power();
				temp = 1;
				while (pow-- > 0) 
				{
					temp *= result;
				}
				result = temp;
			}

			return result;
		}

		private int Factor()  throws ParseException
		{
			//factor -> [-]factor1
			int result;
			if (scanner.CurrentToken().gettkType() == Token.MINUS) 
			{
				Match(Token.MINUS);
				result = - Factor1();
			} 
			else 
			{
				result = Factor1();
			}
			return result;
		}

		private int Factor1()  throws ParseException
		{
			//factor1 -> (expr)|Number
			int result;
			if (scanner.CurrentToken().gettkType() == Token.LPAREN) 
			{
				Match(Token.LPAREN);
				result = Expr();
				Match(Token.RPAREN);
			} 
			else 
			{
				result = scanner.CurrentToken().gettkValue();
				Match(Token.NUMBER);
			}
			return result;
		}
	

}

